@@ -133,6 +133,43 @@ get_pool_config_from_json(char *json_data, int data_len)
 		strncpy(config->backend_desc->backend_info[i].backend_hostname, ptr, sizeof(config->backend_desc->backend_info[i].backend_hostname) - 1);
 	}
 
+	value = json_get_value_for_key(root, "health_check_params");
+	/* We don't get separate health check params from older version
+	 * so be kind if the JSON does not contain one
+	 */
+	if (value != NULL && value->type == json_array)
+	{
+		int health_check_params_count = value->u.array.length;
+		if (health_check_params_count != config->backend_desc->num_backends)
+		{
+			ereport(LOG,
+				(errmsg("unexpected number of health check parameters received"),
+				errdetail("expected:%d got %d",config->backend_desc->num_backends,health_check_params_count)));
+		}
+		config->health_check_params = palloc0(sizeof(HealthCheckParams) * config->backend_desc->num_backends);
+
+		if (health_check_params_count > config->backend_desc->num_backends)
+			health_check_params_count = config->backend_desc->num_backends;
+
+		for (i = 0; i < health_check_params_count; i++)
+		{
+			json_value *arr_value = value->u.array.values[i];
+
+			if (json_get_int_value_for_key(arr_value, "health_check_timeout", &config->health_check_params[i].health_check_timeout))
+				config->health_check_params[i].health_check_timeout = 0;
+			if (json_get_int_value_for_key(arr_value, "health_check_period", &config->health_check_params[i].health_check_period))
+				config->health_check_params[i].health_check_period = 0;
+			if (json_get_int_value_for_key(arr_value, "health_check_max_retries", &config->health_check_params[i].health_check_max_retries))
+				config->health_check_params[i].health_check_max_retries = 0;
+			if (json_get_int_value_for_key(arr_value, "health_check_retry_delay", &config->health_check_params[i].health_check_retry_delay))
+				config->health_check_params[i].health_check_retry_delay = 0;
+			if (json_get_int_value_for_key(arr_value, "connect_timeout", &config->health_check_params[i].connect_timeout))
+				config->health_check_params[i].connect_timeout = 0;
+		}
+	}
+	else
+			config->health_check_params = NULL;
+
 	/* wd_nodes array */
 	value = json_get_value_for_key(root, "wd_nodes");
 	if (value == NULL || value->type != json_array)
@@ -208,6 +245,23 @@ get_pool_config_json(void)
 	jw_put_bool(jNode, "failover_require_consensus", pool_config->failover_require_consensus);
 	jw_put_bool(jNode, "allow_multiple_failover_requests_from_node", pool_config->allow_multiple_failover_requests_from_node);
 
+	/* Array of health_check params
+	 * We transport num_backend at max
+	 */
+	jw_start_array(jNode, "health_check_params");
+	for (i = 0; i < pool_config->backend_desc->num_backends; i++)
+	{
+		jw_start_object(jNode, "HealthCheckParams");
+
+		jw_put_int(jNode, "health_check_timeout", pool_config->health_check_params[i].health_check_timeout);
+		jw_put_int(jNode, "health_check_period", pool_config->health_check_params[i].health_check_period);
+		jw_put_int(jNode, "health_check_max_retries", pool_config->health_check_params[i].health_check_max_retries);
+		jw_put_int(jNode, "health_check_retry_delay", pool_config->health_check_params[i].health_check_retry_delay);
+		jw_put_int(jNode, "connect_timeout", pool_config->health_check_params[i].connect_timeout);
+		jw_end_element(jNode);
+	}
+	jw_end_element(jNode);		/* backend_desc array End */
+
 	/* Array of backends */
 	jw_start_array(jNode, "backend_desc");
 	for (i = 0; i < pool_config->backend_desc->num_backends; i++)
@@ -804,12 +858,12 @@ get_wd_simple_message_json(char *message)
 }
 
 char *
-get_wd_exec_cluster_command_json(char *clusterCommand, int nArgs,
-								 WDExecCommandArg *wdExecCommandArg,
+get_wd_exec_cluster_command_json(char *clusterCommand, List *args_list,
 								 unsigned int sharedKey, char *authKey)
 {
-	int i;
-	char	   *json_str;
+	char	*json_str;
+	int 	nArgs = args_list? list_length(args_list):0;
+
 	JsonNode   *jNode = jw_create_with_object(true);
 
 	jw_put_int(jNode, WD_IPC_SHARED_KEY, sharedKey);	/* put the shared key */
@@ -824,12 +878,15 @@ get_wd_exec_cluster_command_json(char *clusterCommand, int nArgs,
 	/* Array of arguments */
 	if(nArgs > 0)
 	{
+		ListCell   *lc;
 		jw_start_array(jNode, "argument_list");
-		for (i = 0; i < nArgs; i++)
+
+		foreach(lc, args_list)
 		{
+			WDExecCommandArg *wdExecCommandArg = lfirst(lc);
 			jw_start_object(jNode, "Arg");
-			jw_put_string(jNode, "arg_name", wdExecCommandArg[i].arg_name);
-			jw_put_string(jNode, "arg_value", wdExecCommandArg[i].arg_value);
+			jw_put_string(jNode, "arg_name", wdExecCommandArg->arg_name);
+			jw_put_string(jNode, "arg_value", wdExecCommandArg->arg_value);
 			jw_end_element(jNode);
 		}
 		jw_end_element(jNode);		/* argument_list array End */
@@ -844,12 +901,14 @@ get_wd_exec_cluster_command_json(char *clusterCommand, int nArgs,
 
 bool
 parse_wd_exec_cluster_command_json(char *json_data, int data_len,
-								   char **clusterCommand,
-								   int *nArgs, WDExecCommandArg **wdExecCommandArg)
+								   char **clusterCommand, List **args_list)
 {
 	json_value *root;
 	char	*ptr = NULL;
 	int		i;
+	int 	nArgs = 0;
+
+	*args_list = NULL;
 
 	root = json_parse(json_data, data_len);
 
@@ -873,44 +932,50 @@ parse_wd_exec_cluster_command_json(char *json_data, int data_len,
 	}
 	*clusterCommand = pstrdup(ptr);
 
-	if (json_get_int_value_for_key(root, "nArgs", nArgs))
+	if (json_get_int_value_for_key(root, "nArgs", &nArgs))
 	{
 		/* nArgs not found, Just ignore it */
-		*nArgs = 0;
+		nArgs = 0;
 		/* it may be from the old version */
 	}
-	if (*nArgs > 0)
+	if (nArgs > 0)
 	{
 		json_value *value;
-
-		*wdExecCommandArg = palloc0(sizeof(WDExecCommandArg) * *nArgs);
-
 		/* backend_desc array */
 		value = json_get_value_for_key(root, "argument_list");
 		if (value == NULL || value->type != json_array)
 			goto ERROR_EXIT;
 
-		if (*nArgs!= value->u.array.length)
+		if (nArgs!= value->u.array.length)
 		{
 			ereport(LOG,
 					(errmsg("watchdog is unable to parse exec cluster command json"),
 					 errdetail("nArgs is different than argument array length \"%s\"", json_data)));
 			goto ERROR_EXIT;
 		}
-		for (i = 0; i < *nArgs; i++)
+		for (i = 0; i < nArgs; i++)
 		{
+			WDExecCommandArg *command_arg = palloc0(sizeof(WDExecCommandArg));
+			/*
+			 * Append to list right away, so that deep freeing the list also
+			 * get rid of half cooked arguments in case of an error
+			 */
+			*args_list = lappend(*args_list,command_arg);
+
 			json_value *arr_value = value->u.array.values[i];
 			char	   *ptr;
 
 			ptr = json_get_string_value_for_key(arr_value, "arg_name");
 			if (ptr == NULL)
 				goto ERROR_EXIT;
-			strncpy(wdExecCommandArg[i]->arg_name, ptr, sizeof(wdExecCommandArg[i]->arg_name) - 1);
+
+			strncpy(command_arg->arg_name, ptr, sizeof(command_arg->arg_name) - 1);
 
 			ptr = json_get_string_value_for_key(arr_value, "arg_value");
 			if (ptr == NULL)
 				goto ERROR_EXIT;
-			strncpy(wdExecCommandArg[i]->arg_value, ptr, sizeof(wdExecCommandArg[i]->arg_value) - 1);
+
+			strncpy(command_arg->arg_value, ptr, sizeof(command_arg->arg_value) - 1);
 		}
 	}
 
@@ -920,7 +985,8 @@ parse_wd_exec_cluster_command_json(char *json_data, int data_len,
 	ERROR_EXIT:
 	if (root)
 		json_value_free(root);
-	if (*wdExecCommandArg )
-		pfree(*wdExecCommandArg);
+	if (*args_list)
+		list_free_deep(*args_list);
+	*args_list = NULL;
 	return false;
 }
